/*==========================================================================*\
|                                                                            |
| Replicator.c                                                               |
|                                                                            |
| JTAG Replicator for the MSP430 Flash-based family devices                  |
|                                                                            |
| Key features:                                                              |
|  Supports JTAG communication to all MSP430 Flash Devices                  |
|  Max. code size of target program: 57kB                                   |
|  Programming speed: ~60kB/10sec (~6kB/sec)                                |
|  Fast Verification and Erase Check: ~60kB/350ms                           |
|  Supports Programming of JTAG Access Protection Fuse                      |
|                                                                            |
|----------------------------------------------------------------------------|
| Project:              MSP430 Replicator                                    |
| Developed using:      IAR Embedded Workbench 3.40B [Kickstart]             |
|----------------------------------------------------------------------------|
| Author:               FRGR                                                 |
| Version:              1.8                                                  |
| Initial Version:      04-17-02                                             |
| Last Change:          09-21-05                                             |
|----------------------------------------------------------------------------|
| Version history:                                                           |
| 1.0 04/02 FRGR        Initial version.                                     |
| 1.1 04/02 FRGR        Included SPI mode to speed up shifting function by 2.|
| 1.2 06/02 ALB2        Formatting changes, added comments.                  |
| 1.3 06/02 ALB2        DEVICE ID info added.                                |
| 1.4 08/02 ALB2        Initial code release with Lit# SLAA149.              |
| 1.5 01/03 ALB2        Unnecessary lines have been commented out(see below).|
| 1.6 09/05 JDI         converted to IAR EW 3.30. F2xxx support added        |
|           SUN1        Software delays redesigned to use TimerA harware;    |
|                       see MsDelay() routine.                               |
|           ALB2        Added 2xx Info memory B, C, & D erase & check        |
| 1.7 12/05 STO         Adapted for 2xx devices with SpyBiWire               |
| 1.8 02/06 STO         Minor cosmetic changes                               |
|----------------------------------------------------------------------------|
| Designed 2002 by Texas Instruments Germany                                 |
\*==========================================================================*/

// THIS PROGRAM IS PROVIDED "AS IS". TI MAKES NO WARRANTIES OR
// REPRESENTATIONS, EITHER EXPRESS, IMPLIED OR STATUTORY,
// INCLUDING ANY IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS
// FOR A PARTICULAR PURPOSE, LACK OF VIRUSES, ACCURACY OR
// COMPLETENESS OF RESPONSES, RESULTS AND LACK OF NEGLIGENCE.
// TI DISCLAIMS ANY WARRANTY OF TITLE, QUIET ENJOYMENT, QUIET
// POSSESSION, AND NON-INFRINGEMENT OF ANY THIRD PARTY
// INTELLECTUAL PROPERTY RIGHTS WITH REGARD TO THE PROGRAM OR
// YOUR USE OF THE PROGRAM.
//
// IN NO EVENT SHALL TI BE LIABLE FOR ANY SPECIAL, INCIDENTAL,
// CONSEQUENTIAL OR INDIRECT DAMAGES, HOWEVER CAUSED, ON ANY
// THEORY OF LIABILITY AND WHETHER OR NOT TI HAS BEEN ADVISED
// OF THE POSSIBILITY OF SUCH DAMAGES, ARISING IN ANY WAY OUT
// OF THIS AGREEMENT, THE PROGRAM, OR YOUR USE OF THE PROGRAM.
// EXCLUDED DAMAGES INCLUDE, BUT ARE NOT LIMITED TO, COST OF
// REMOVAL OR REINSTALLATION, COMPUTER TIME, LABOR COSTS, LOSS
// OF GOODWILL, LOSS OF PROFITS, LOSS OF SAVINGS, OR LOSS OF
// USE OR INTERRUPTION OF BUSINESS. IN NO EVENT WILL TI'S
// AGGREGATE LIABILITY UNDER THIS AGREEMENT OR ARISING OUT OF
// YOUR USE OF THE PROGRAM EXCEED FIVE HUNDRED DOLLARS
// (U.S.$500).
//
// Unless otherwise stated, the Program written and copyrighted
// by Texas Instruments is distributed as "freeware".  You may,
// only under TI's copyright in the Program, use and modify the
// Program without any charge or restriction.  You may
// distribute to third parties, provided that you transfer a
// copy of this license to the third party and the third party
// agrees to these terms by its first use of the Program. You
// must reproduce the copyright notice and any other legend of
// ownership on each copy or partial copy, of the Program.
//
// You acknowledge and agree that the Program contains
// copyrighted material, trade secrets and other TI proprietary
// information and is protected by copyright laws,
// international copyright treaties, and trade secret laws, as
// well as other intellectual property laws.  To protect TI's
// rights in the Program, you agree not to decompile, reverse
// engineer, disassemble or otherwise translate any object code
// versions of the Program to a human-readable form.  You agree
// that in no event will you alter, remove or destroy any
// copyright notice included in the Program.  TI reserves all
// rights not specifically granted under this license. Except
// as specifically provided herein, nothing in this agreement
// shall be construed as conferring by implication, estoppel,
// or otherwise, upon you, any license or other right under any
// TI patents, copyrights or trade secrets.
//

/****************************************************************************/
/* Includes                                                                 */
/****************************************************************************/

#include "JTAGfunc.h"            // JTAG functions
#include "LowLevelFunc.h"        // low level user functions
#include "Devices.h"            // holds Device specific information


/****************************************************************************/
/* Global types                                                             */
/****************************************************************************/

extern word CodeArray[10];        // Holds the target program code, index 10 only for debug

/****************************************************************************/
/* Main section of Replicator program: User can modify/insert code as needed*/
/****************************************************************************/
/* Main function:
   Upon execution completion, LED blinks for F1xx & F4xx device FET tools
   when used as the target system(P1.0 and P5.1 drive LEDs).
   The target program is stored in file "Target_Code.s43" (see "CodeArray[]")
   which must be added to the project as an Assembler source code file.

   Note: All High Level JTAG Functions are applied here.

   A number of lines have been commented out which are not strictly required
   to successfully program a flash-based MSP430 device using the Replicator
   concept. Please note that these lines have been commented out in order
   to provide reference examples for calling such functions.
   Uncommented lines below represent the minimum requirements to successfully
   program an MSP430 flash device using the device Replicator.
*/

void main(void)
{/* The two variables below are used only in the WriteMemQuick and ReadMemQuick
    functions. These function calls have been commented out below. Uncomment these
    variable declarations if using the respective routines. */
  word p;                               // Data pointer
  word ReadArray[0x40];                 // Buffer, used for ReadMemQuick()

  InitController();                     // Initialize the Host 'F149
  ShowStatus(STATUS_ACTIVE, 0);         // Switch both LEDs on to indicate
                                        // operation.

  InitTarget();                         // Initialize Target Board
  if (!GetDevice())                     // Set DEVICE index, stop
    ShowStatus(STATUS_ERROR, 1);        // here if invalid JTAG ID or
                                        // time-out. (error: red LED is ON)

// The following section is not required and included only for reference as to the
// manner in which a single flash memory segment can be erased. The more common
// "mass erase" is used to prepare the target device for replicator programming.

  //EraseFLASH(ERASE_SGMT, 0xFE00);       // Segment 0 erase Flash (all types)
  //if (!EraseCheck(0xFE00, 0x0100))      // Check segment 0 memory erasure (Fxx2..9)
  //  ShowStatus(STATUS_ERROR, 2);

  if (DeviceHas_CpuX())
  {
    EraseFLASH(ERASE_GLOB, 0xFE00);       // Mass-Erase Flash (all types)
  }
  else
  {
    EraseFLASH(ERASE_MASS, 0xFE00);       // Mass-Erase Flash (all types)
  }

  if (!EraseCheck(0x1000, 0x0040))      // Check main memory erasure (Fxx2..9)
   ShowStatus(STATUS_ERROR, 3);

  if (!EraseCheck(0xF800, 0x0400))      // Check main memory erasure (Fxx2..9)
   ShowStatus(STATUS_ERROR, 3);

/* For all 1xx & 4xx & 2xx devices, where ALL Flash has been erased*/
  EraseFLASH(ERASE_SGMT, 0xFE00);
  if (!EraseCheck(0xfe00, 0x0100))      // Check part of main memory erasure (Fxx2..9)
    ShowStatus(STATUS_ERROR, 4);
/* For all 2xx devices, where Info A was not erased (Info A keeps calibration data) */
  //if (!EraseCheck(0x1000, 0x0080))      // Check info memory erasure (Fxx2..9)
  //  ShowStatus(STATUS_ERROR, 4);

/* // Following section shows how to erase Info-Segments on 2xx Devices selectively
  EraseFLASH(ERASE_SGMT, 0x1080);       // Info B erase (2xx)
  if (!EraseCheck(0x1080, 0x0020))      // Check Info B memory erasure (2xx)
    ShowStatus(STATUS_ERROR, 2);
  EraseFLASH(ERASE_SGMT, 0x1040);       // Info C erase (2xx)
  if (!EraseCheck(0x1040, 0x0020))      // Check Info C memory erasure (2xx)
    ShowStatus(STATUS_ERROR, 2);
  EraseFLASH(ERASE_SGMT, 0x1000);       // Info D erase (2xx)
  if (!EraseCheck(0x1000, 0x0020))      // Check Info D memory erasure (2xx)
    ShowStatus(STATUS_ERROR, 2);
  // Info A left as it contains calibration data
*/

// The following section is not required in order to program the Flash memory
// of a flash-based MSP430 device.
// These lines are shown only as examples for the purpose of understanding all
// functions included in this software package.
//

  WriteMem(F_BYTE, 0x0200, 0x34);     // Communication test: Write 2 RAM bytes,
  WriteMem(F_BYTE, 0x0201, 0x12);       // read back word
  if (ReadMem(F_WORD, 0x0200) != 0x1234)
    ShowStatus(STATUS_ERROR, 5);
  WriteMem(F_WORD, 0x0202, 0x5678);     // Write RAM word, read back 2 bytes
  if (ReadMem(F_BYTE, 0x0202) != 0x78)
    ShowStatus(STATUS_ERROR, 6);
  if (ReadMem(F_BYTE, 0x0203) != 0x56)
    ShowStatus(STATUS_ERROR, 7);

  // Is the Read/WriteQuick for current device possible?
  if (DeviceHas_DataQuick())
  {
    WriteMemQuick (0x0200, 0x0010, &CodeArray[0]);    // Write RAM block
      if (!VerifyMem(0x0200, 0x0010, &CodeArray[0]))    // Verify(PSA) RAM of
        ShowStatus(STATUS_ERROR, 8);                  // target.
      ReadMemQuick  (0x0200, 0x0010, &ReadArray[0]);    // Read RAM block
      for (p = 0; p < 0x0010; p++)            // Verify(word-for-
         if (ReadArray[p] != CodeArray[p])             // word) RAM of target.
          ShowStatus(STATUS_ERROR, 9);
  }

  if (!WriteFLASHallSections(&CodeArray[0]))        // Write/Verify(PSA)
    ShowStatus(STATUS_ERROR, 10);                       // all Flash sections.

// The following lines have been commented out as they are not required for
// general programming of an MSP430 flash-based device. They are included
// as a method of indicating successful programming of an MSP430 FET kit board.
// These lines flash the LED of the target board upon programming completion.

#if 0
  WriteMem(F_BYTE, P1OUT_, 0x01);       // Control target peripherals directly:
  WriteMem(F_BYTE, P5OUT_, 0x02);       // P1.0 for F1xx,2xx devices
  WriteMem(F_BYTE, P1DIR_, 0x01);       // P5.1 for F4xx devices
  WriteMem(F_BYTE, P5DIR_, 0x02);
  MsDelay(1000);                        // LED on for 1.0s
  WriteMem(F_BYTE, P1OUT_, 0x00);
  WriteMem(F_BYTE, P5OUT_, 0x00);
  MsDelay(1000);                        // LED off for 1.0s
#endif

  /*****************************************************************************
  Remove following comments to enable fuse blow routine. Blowing of the JTAG
  access protection fuse is irreversible.*/

  //if (!BlowFuse())                    // Blow security fuse
  //  ShowStatus(STATUS_ERROR, 15);     // ***Action is permanent***

  /****************************************************************************/

  ReleaseDevice(V_RESET);               // Perform Reset, release CPU from JTAG control
  MsDelay(3000);                        // Target application blinks for 3.0s
  ReleaseTarget();                      // Release Target Board from power

  ShowStatus(STATUS_OK, 0);             // OK: green LED is ON
  while(1);                             // Idle, wait for 'GO' keypress
}

/****************************************************************************/
/*                         END OF SOURCE FILE                               */
/****************************************************************************/
